#include "vector.h"
#include <stdlib.h>
#include <assert.h>

static const int StartSize = 5;       /* initial vector capacity */
static const float GrowthRate = 1.8f; /* geometric growth of vector capacity */

Vector create_vector(int capacity)
/* Initialise a vector with the specified capacity. */
{
    Vector v;
    v.size = 0;
    v.data = (ValueType *)malloc(capacity*sizeof(ValueType));
    v.capacity = (v.data == NULL) ? 0 : capacity;
    return v;
}

void release_vector(Vector *v)
/* Release memory owned by vector. */
{
    free(v->data);
    v->size = 0;
    v->capacity = 0;
}

int push_back(Vector *v, ValueType item)
/* Add element to back of vector. Return index of new element if successful, and -1 if fails. */
{
    /* If out-of-space, allocate more. */
    if (v->size == v->capacity) { 
        int newsize = (v->capacity == 0) ? StartSize : (int)(v->capacity*GrowthRate + 1.0); 
        ValueType *p = (ValueType *)realloc(v->data, newsize*sizeof(ValueType));
        if (p == NULL)
            return -1;

        v->capacity = newsize; /* allocate succeeds, update data-structure */
        v->data = p;
    }

    /* We have enough room. */
    v->data[v->size] = item;
    return v->size++;
}

ValueType pop_back(Vector *v)
/* Return element from back of vector, and remove it from the vector. */
{
    assert(v->size > 0);
    return v->data[--v->size];
}

ValueType* get_element(Vector *v, int index)
/* Return pointer to the element at the specified index. */
{
    assert(index >= 0 && index < v->size);
    return v->data + index;
}

/* Manual size operations. */
int get_size(Vector *v) { return v->size; }
int get_capacity(Vector *v) { return v->capacity; }

int set_size(Vector *v, int size)
/* Set vector size. Return 0 if successful, -1 if fails. */
{
    if (size > v->capacity) {
        ValueType *p = (ValueType *)realloc(v->data, size*sizeof(ValueType));
        if (p == NULL)
            return -1;

        v->capacity = size; /* allocate succeeds, update data-structure */
        v->data = p;
    }

    v->size = size;
    return 0;
}

int set_capacity(Vector *v, int size)
/* Shrink or grow allocated memory reserve for array.
 * A size of 0 deletes the array. Return 0 if successful, -1 if fails. */
{
    if (size != v->capacity) {
        ValueType *p = (ValueType *)realloc(v->data, size*sizeof(ValueType));
        if (p == NULL && size > 0)
            return -1;

        v->capacity = size;
        v->data = p;
    }

    if (size < v->size)
        v->size = size;
    return 0;
}
